<?php

/*
Copyright 2016-2016 Amazon.com, Inc. or its affiliates. All Rights Reserved.

Licensed under the GNU General Public License as published by the Free Software Foundation,
Version 2.0 (the "License"). You may not use this file except in compliance with the License.
A copy of the License is located in the "license" file accompanying this file.

This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
either express or implied. See the License for the specific language governing permissions
and limitations under the License.
*/

/**
 * The class that loads all the hooks.
 *
 * @since      1.0.0
 * @package    AmazonAssociatesLinkBuilder
 * @subpackage AmazonAssociatesLinkBuilder/includes
 */
class Aalb_Hook_Loader {

  protected $actions;

  public function __construct() {
    $this->actions = array();
  }

  /**
   * Add a new action to the collection to be registered with WordPress.
   *
   * @since    1.0.0
   * @param    string    $hook             The name of the WordPress action that is being registered.
   * @param    object    $component        A reference to the instance of the object on which the action is defined.
   * @param    string    $callback         The name of the function definition on the $component.
   * @param    int       $priority         Optional. he priority at which the function should be fired. Default is 10.
   * @param    int       $accepted_args    Optional. The number of arguments that should be passed to the $callback. Default is 1.
   */
  public function add_action( $hook, $component, $callback, $priority = 10, $accepted_args = 1 ) {
    $this->actions = $this->add( $this->actions, $hook, $component, $callback, $priority, $accepted_args );
  }

  /**
   * A utility function that is used to register the actions and hooks into a single
   * collection.
   *
   * @since     1.0.0
   * @access    private
   * @param     array     $hooks            The collection of hooks that is being registered (that is, actions or filters).
   * @param     string    $hook             The name of the WordPress filter that is being registered.
   * @param     object    $component        A reference to the instance of the object on which the filter is defined.
   * @param     string    $callback         The name of the function definition on the $component.
   * @param     int       $priority         The priority at which the function should be fired.
   * @param     int       $accepted_args    The number of arguments that should be passed to the $callback.
   * @return    array                       The collection of actions and filters registered with WordPress.
   */
  private function add( $hooks, $hook, $component, $callback, $priority, $accepted_args ) {
    $hooks[] = array(
      'hook'          => $hook,
      'component'     => $component,
      'callback'      => $callback,
      'priority'      => $priority,
      'accepted_args' => $accepted_args
    );
    return $hooks;
  }

  /**
   * Register the filters and actions with WordPress.
   *
   * @since    1.0.0
   */
  public function execute() {
    foreach ( $this->actions as $hook ) {
      add_action( $hook['hook'], array( $hook['component'], $hook['callback'] ), $hook['priority'], $hook['accepted_args'] );
    }
  }

}

?>
